/* 
 * Keith J O'Hara <kohara@bard.edu.
 * January 2013
 *
 * sp5repl 
 * A Scala REPL for the Processsing Programming Environment.
 *
 */

package sp5repl

import math._
import scala.util.Random
import processing.core.PApplet
import scala.tools.nsc.interpreter._
import java.io._

object sp5repl extends PApplet{
  
  var sketch:sp5repl = _
  
  def main(args: Array[String]) = {
    sketch = createSketch()
    sketch.startInterpreter(args)
  }
   
  def createSketch(width: Int = 500, height:Int = 500): sp5repl= {
    val sketch = new sp5repl
    sketch.startSketch(width, height)
    sketch.size(width, height)
    sketch
  }
}

class sp5repl extends PApplet (){

  implicit def float2Double(f: Float) = f.toDouble
  implicit def double2Float(d: Double) = d.toFloat
  type color = Int
  
  var _frame: javax.swing.JFrame = null
  var _setup: () => Unit = null
  var _draw: () => Unit = null
  var _keyPressed: () => Unit = null
  var _keyReleased: () => Unit = null
  var _mousePressed: () => Unit = null
  var _mouseDragged: () => Unit = null
  var _mouseReleased: () => Unit = null
  var _mouseMoved: () => Unit = null
  var noCompletion = false

  var il:ILoop = _

  def startSketch(w: Int = 500, h:Int = 500) = {
    _frame = new javax.swing.JFrame("sp5repl")
    _frame.getContentPane().add(this)
    _frame.setResizable(true)
    _frame.pack
    width = w
    height = h
    init
    while (defaultSize && !finished)  delay(5)
    _frame.setVisible(true)    
  }

  override def size(w: Int, h: Int) = {
    if (_frame != null) _frame.setSize(w, h)
    super.size(w, h)
  }
  
  override def setup() = if (_setup != null) _setup()  
  override def draw() = if (_draw != null) _draw()
  override def keyPressed() = if (_keyPressed != null) _keyPressed()
  override def keyReleased() = if (_keyReleased != null) _keyReleased()
  override def mousePressed() = if (_mousePressed != null) _mousePressed()
  override def mouseDragged() = if (_mouseDragged != null) _mouseDragged()
  override def mouseReleased() = if (_mouseReleased != null) _mouseReleased()
  override def mouseMoved() = if (_mouseMoved != null) _mouseMoved()

  def onDraw (f: ()=> Unit) = _draw = f
  def onDraw() = _draw = null
  
  def onMousePressed (f: () => Unit) =  _mousePressed = f
  def onMousePressed () =  _mousePressed = null

  def onMouseDragged (f: () => Unit) = _mouseDragged = f
  def onMouseDragged () =  _mouseDragged = null

  def onMouseReleased (f: () => Unit) = _mouseReleased = f
  def onMouseReleased () = _mouseReleased = null

  def onMouseMoved (f: () => Unit) = _mouseMoved = f
  def onMouseMoved () = _mouseMoved = null

  def onKeyPressed (f: () => Unit) = _keyPressed = f
  def onKeyPressed () = _keyPressed = null

  def onKeyReleased (f: () => Unit) =  _keyReleased = f
  def onKeyReleased () = _keyReleased = null

  def createSketch(w: Int = 500, h:Int = 500): sp5repl = { 
    sp5repl.createSketch(w, h)
  }

  def newSketch(w: Int = 500, h:Int = 500) = { 
    val s = sp5repl.createSketch(w, h)
    s.il = il
    il.bind("sketch", s)
    il.quietRun("""import sketch._""")
  }

  def reset() = {
    il.reset()
    il.bind("sketch", this)    
    il.quietRun("""
		  import processing.core._
		  import processing.core.PConstants._
          import processing.core.PApplet._
		  import sp5repl.sp5repl
		  import sketch._""")
  }

  def startInterpreter(args: Array[String]) = {
    
    val out = new PrintWriter(new BufferedWriter(
      new OutputStreamWriter(System.out)))  
    
    if (args.contains("-t")) noCompletion = true
    
    val sketch: sp5repl = this;

    il = new ILoop {
      override def createInterpreter(){
	      try {
            super.createInterpreter()
	        intp.quietBind("sketch", sketch)
     	    intp.quietRun("""
                          import processing.core._
                          import processing.core.PConstants._
                          import processing.core.PApplet._
                          import sp5repl.sp5repl			     
			              import sketch._""")
            
  	        for (f <- args if f != "-t"){
	          val inputFile = io.Source.fromFile(f).mkString
	            intp.quietRun(inputFile)	    
	        }      
	     }  
      }
    }
    
    il.setPrompt("sp5repl>")
    il.settings = new scala.tools.nsc.Settings()    
    il.settings.embeddedDefaults(getClass.getClassLoader)
    il.settings.usejavacp.value = true
    il.settings.noCompletion.value = noCompletion
    // don't put each repl line in new thread; conflicts with bind
    il.settings.Yreplsync.value = true  
  
    il.process(il.settings)
    println("Good bye");
    il.closeInterpreter()    
  }
  
}
